<?php
require_once("Condition.php");

class Doctrine_Query_Where extends Doctrine_Query_Condition {
    /**
     * load
     * returns the parsed query part
     *
     * @param string $where
     * @return string
     */
    public function load($where) {
        $where = trim($where);

        $e     = Doctrine_Query::sqlExplode($where);

        if(count($e) > 1) {
            $tmp   = $e[0].' '.$e[1];

            if(substr($tmp, 0, 6) == 'EXISTS')
                return $this->parseExists($where, true);
            elseif(substr($where, 0, 10) == 'NOT EXISTS')
                return $this->parseExists($where, false);
        }

        if(count($e) < 3) {
            $e = Doctrine_Query::sqlExplode($where, array('=', '<', '>', '!='));
        }
        $r = array_shift($e);

        $a = explode(".",$r);

        if(count($a) > 1) {
            $field     = array_pop($a);
            $count     = count($e);
            $slice     = array_slice($e, -1, 1);
            $value     = implode('', $slice);
            $operator  = trim(substr($where, strlen($r), -strlen($value)));

            $reference = implode(".",$a);
            $count     = count($a);



            $pos       = strpos($field, "(");

            if($pos !== false) {
                $func   = substr($field, 0, $pos);
                $value  = trim(substr($field, ($pos + 1), -1));

                $values = Doctrine_Query::sqlExplode($value, ',');

                $field      = array_pop($a);
                $reference  = implode(".",$a);
                $table      = $this->query->load($reference, false);
                array_pop($a);
                $reference2 = implode('.', $a);
                $alias      = $this->query->getTableAlias($reference2);

                $stack      = $this->query->getRelationStack();
                $relation   = end($stack);
                
                $stack      = $this->query->getTableStack();

                switch($func) {
                    case 'contains':
                    case 'regexp':
                    case 'like':
                        $operator = $this->getOperator($func);

                        if(empty($relation))
                            throw new Doctrine_Query_Exception('DQL functions contains/regexp/like can only be used for fields of related components');
                        
                        $where = array();
                        foreach($values as $value) {
                            $where[] = $alias.'.'.$relation->getLocal().
                              ' IN (SELECT '.$relation->getForeign().
                              ' FROM '.$relation->getTable()->getTableName().' WHERE '.$field.$operator.$value.')';
                        }
                        $where = implode(' AND ', $where);
                    break;
                    default:
                        throw new Doctrine_Query_Exception('Unknown DQL function: '.$func);
                }
            } else {
                $table     = $this->query->load($reference, false);
                $enumIndex = $table->enumIndex($field, trim($value,"'"));
                $alias     = $this->query->getTableAlias($reference);
                $table     = $this->query->getTable($alias);


                if($value == 'true')
                    $value = 1;
                elseif($value == 'false')
                    $value = 0;
                elseif(substr($value,0,5) == '(FROM') {
                    // subquery
                    $sub   = Doctrine_Query::bracketTrim($value);
                    $q     = new Doctrine_Query();
                    $value = '(' . $q->parseQuery($sub)->getQuery() . ')';
                } else {
                    // check that value isn't a string
                    if(strpos($value, '\'') === false) {
                        $a = explode('.', $value);

                        if(count($a) > 1) {
                            // either a float or a component..
    
                            if( ! is_numeric($a[0])) {
                                // a component found
                                $value = $this->query->getTableAlias($a[0]). '.' . $a[1];
                            }
                        }
                    }
                }

                switch($operator) {
                    case '<':
                    case '>':
                    case '=':
                    case '!=':
                        if($enumIndex !== false)
                            $value  = $enumIndex;
                    default:
                        $where      = $alias.'.'.$field.' '.$operator.' '.$value;
                }
            }
        }
        return $where;
    }

    public function parseExists($where, $negation) {
        $operator = ($negation) ? 'EXISTS' : 'NOT EXISTS';

        $pos = strpos($where, '(');
        
        if($pos == false)
            throw new Doctrine_Query_Exception("Unknown expression, expected '('");

        $sub = Doctrine_Query::bracketTrim(substr($where, $pos));

        return $operator . ' ('.$this->query->createSubquery()->parseQuery($sub, false)->getQuery() . ')';
    }

    public function getOperator($func) {
        switch($func) {
            case 'contains':
                $operator = ' = ';
            break;
            case 'regexp':
                $operator = $this->query->getConnection()->getRegexpOperator();
            break;
            case 'like':
                $operator = ' LIKE ';
            break;
        }
        return $operator;
    }

    public function __toString() {
        return ( ! empty($this->parts))?implode(" AND ", $this->parts):'';
    }
}

